<!-- BEGIN block_one -->
{glob}:{var1}#
<!-- END block_one -->
<!-- BEGIN block_two -->
{glob}:{var2}#
<!-- END block_two -->
<!-- BEGIN block_three -->
{glob}:{var3}
    <!-- BEGIN block_four -->
    |{glob}:{var4}
    <!-- END block_four -->
#
<!-- END block_three -->
