#ifndef BISON_ZEND_ZEND_INI_PARSER_H
# define BISON_ZEND_ZEND_INI_PARSER_H

# ifndef YYSTYPE
#  define YYSTYPE int
#  define YYSTYPE_IS_TRIVIAL 1
# endif
# define	TC_STRING	257
# define	TC_ENCAPSULATED_STRING	258
# define	SECTION	259
# define	CFG_TRUE	260
# define	CFG_FALSE	261


#endif /* not BISON_ZEND_ZEND_INI_PARSER_H */
